#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <rfftw.h>
#include <sys/io.h>
#include <math.h>

#include "io.h"
#include "chrtab.h"

static unsigned tab[768];

static void Die(int dummy)
{
    int a;
    outportb(0x3C8, 0);
    for(a=0; a<768; ++a)outportb(0x3C9, tab[a]);
    dummy = dummy;
    fprintf(stderr, "Kuolema ja kirous!\n");
    signal(dummy, SIG_DFL);
    raise(dummy);
//  raise(SIGKILL);
}

/* Samplaustaajuus */
#define FREQ    44100

/* Kuinka iso palanen kerralla skannataan */
#define N        10000

/* Kuinka monta skannausta samalle palalle voidaan tehd (askellus) */
#define PALASIA  80

#define Step    ((FREQ / PALASIA)<N?(FREQ/PALASIA):N)
                /* Stepin pit olla <= N, muuten tippuu jotain
                 * vlist. Jotenka siis l valitse pienempi
                 * arvoja PALASIA-kohtaan kuin FREQ/N, muuten
                 * tulos ei ole mit odotit.
                 *
                 * Valitse mielelln kuitenkin isompi arvo
                 * kuin FREQ/N, sill muuten ohjelma blokkaa
                 * odottaessaan uutta dataa, mik nkyy nykimisen.
                 */

typedef signed short intype;

#define inadd 32768
#define inmax 65536

static fftw_real In[N];
static fftw_real Power[N/2+1];

static fftw_plan Plan;

static intype Buf[N];

static int piippu(void)
{
    int p[2], i;
    if(pipe(p) < 0)
    {
        fprintf(stderr, "Piippu mokasi!\n");
        exit(-1);
    }
    
    i = fork();
    if(i < 0)
    {
        fprintf(stderr, "Haarukka mokasi!\n");
        exit(-1);
    }
    
    if(!i)
    {
        close(p[0]);
        dup2(p[1], 1);
        if(execl("/usr/local/bin/esdrec", "esdrec", "-m", NULL) < 0)
            fprintf(stderr, "Teloitus mokasi!\n");
        exit(-1);
    }
    
    close(p[1]);    
    return p[0];
}

int main(void)
{
    int a;
    int fd = piippu();
    
    int AvgCount;    
    fftw_real Avg[N/2];
    
    Use8bit = 1;    
    
    MkTab();
    
    if(ioperm(0x3C7, 3, 1) < 0)
    {
        fprintf(stderr, "No rights!\n");
        return 0;
    }
    outportb(0x3C7, 0);
    for(a=0; a<768; ++a)tab[a] = inportb(0x3C9);    
    
    signal(SIGTERM, Die);
    signal(SIGINT, Die);
    signal(SIGSEGV, Die);
    
    Plan = rfftw_create_plan(N, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
    
    for(a=0; a<N/2; ++a)Avg[a] = 0;
    AvgCount = 0;
    
    read(fd, &Buf[0], (N)*(sizeof(intype)));
    for(;;)
    {
        fftw_real Out[N];
        rfftw_one(Plan, In, Out);

        Power[0] = fabs(Out[0]);
        for(a=1; a < (N+1)/2; a++)Power[a] = sqrt(Out[a]*Out[a] + Out[N-a]*Out[N-a]);
        if(!(N%2))Power[N/2] = sqrt(Out[N/2]*Out[N/2]);
        
        AvgCount++;
        for(a=0; a<N/2; ++a)
        {
            fftw_real tmp = Power[a];
            Power[a] = fabs(Power[a] - Avg[a] / AvgCount);
            Avg[a] = Avg[a] + tmp;
        }
        
        outportb(0x3C8, 0);        
        for(a=0; a<3; ++a)
        {
            static const int atab[3] = {2,1,0};
            int g = atab[a];
            int b = (g  )*(N/2) / 4 + 1;
            int c = (g+1)*(N/2) / 4;
            int d, val;
            
            double e=0;
            for(d=b; d<c; ++d)
                e += Power[d];
                
//            e /= 10000;
            
            if(a == 0)e *= 2;
            else if(a == 2)e /= 10;
            
            fprintf(stderr, "%10.3f for %d\n", e, c-b);

            val = (int)(e*255);
            if(val > 255)val = 255;
            if(val < 0)val = 0;
                
            outportb(0x3C9, val);
        }
        
        fprintf(stderr, "-----------\n");
        
        /* Read more input data */
        if(!read(fd, &Buf[N-Step], (Step)*(sizeof(intype))))break;
        
        for(a=0; a<Step; a++)
            In[N-Step+a] = (Buf[N-Step+a]+inadd)/(fftw_real)inmax;
    }    
    
    return 0;
}
